@@ -314,7 +314,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 {
     UIViewController<VLCPlaybackServiceDelegate> *movieViewController = self.movieViewController;
     UINavigationController *navCon = [[VLCPlaybackNavigationController alloc] initWithRootViewController:movieViewController];
-    navCon.modalPresentationStyle = UIModalPresentationOverCurrentContext;
+    navCon.modalPresentationStyle = UIModalPresentationOverFullScreen;
     [movieViewController prepareForMediaPlayback:self.playbackController];
 
     UIWindow *window = [[UIApplication sharedApplication] keyWindow];
@@ -327,7 +327,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 {
     UIViewController<VLCPlaybackServiceDelegate> *audioPlayerViewController = self.audioPlayerViewController;
     UINavigationController *navCon = [[VLCPlaybackNavigationController alloc] initWithRootViewController:audioPlayerViewController];
-    navCon.modalPresentationStyle = UIModalPresentationOverCurrentContext;
+    navCon.modalPresentationStyle = UIModalPresentationOverFullScreen;
     [audioPlayerViewController prepareForMediaPlayback:self.playbackController];
 
     UIWindow *window = [[UIApplication sharedApplication] keyWindow];
@@ -464,8 +464,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
                 [[NSNotificationCenter defaultCenter]
                  postNotificationName:VLCPlayerDisplayControllerHideMiniPlayer object:self];
             }
-            [self->_queueViewController hide];
-            [self->_queueViewController removeFromParentViewController];
+
             [self resignFirstResponder];
         };
 
@@ -484,6 +483,8 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 
 - (void)addPlayqueueToMiniPlayer
 {
+    [_queueViewController hide];
+    [_queueViewController removeFromParentViewController];
     [_queueViewController didMoveToParentViewController:self];
     [((VLCAudioMiniPlayer*)_miniPlaybackView) setupQueueViewControllerWith:_queueViewController];
     [self becomeFirstResponder];
