@@ -5,13 +5,14 @@
  *
  * Authors: Soomin Lee <bubu@mikan.io>
  *          Edgar Fouillet <vlc # edgar.fouillet.eu>
+ *          Diogo Simao Marques <dogo@videolabs.io>
  *
  * Refer to the COPYING file of the official project for license.
  *****************************************************************************/
 
 @objc(VLCQueueViewControllerDelegate)
 protocol QueueViewControllerDelegate {
-    func queueViewControllerDidDisappear(_ queueViewController: QueueViewController?)
+    @objc optional func queueViewControllerDidDisappear(_ queueViewController: QueueViewController?)
 }
 
 class QueueViewFlowLayout: UICollectionViewFlowLayout {
@@ -110,7 +111,9 @@ class QueueViewController: UIViewController {
 
     override func viewWillDisappear(_ animated: Bool) {
         super.viewWillDisappear(animated)
-        delegate?.queueViewControllerDidDisappear(self)
+        if let delegate = delegate as? VideoPlayerViewController {
+            delegate.queueViewControllerDidDisappear(self)
+        }
     }
 
     override func viewDidDisappear(_ animated: Bool) {
@@ -134,6 +137,7 @@ class QueueViewController: UIViewController {
             ]
 
             let heightConstraint: NSLayoutConstraint?
+            var miniPlayerView: AudioMiniPlayer? = nil
             if let parent = parent as? VLCPlayerDisplayController, let miniPlaybackView = parent.miniPlaybackView as? AudioMiniPlayer {
                 grabberView.isHidden = true
                 closeButton.isHidden = true
@@ -141,6 +145,7 @@ class QueueViewController: UIViewController {
                 topConstraint = view.topAnchor.constraint(equalTo: miniPlaybackView.bottomAnchor)
                 heightConstraint = nil
                 bottomConstraint = view.bottomAnchor.constraint(equalTo: parent.view.bottomAnchor)
+                miniPlayerView = miniPlaybackView
             } else {
                 grabberView.isHidden = false
                 closeButton.isHidden = false
@@ -150,16 +155,42 @@ class QueueViewController: UIViewController {
                                                                     constant: -(topConstraintConstant + parent.videoPlayerControls.frame.height + parent.scrubProgressBar.frame.height))
                     bottomConstraint = view.bottomAnchor.constraint(equalTo: parent.view.bottomAnchor,
                                                                     constant: self.view.frame.height)
+                } else if let parent = parent as? AudioPlayerViewController {
+                    heightConstraint = view.heightAnchor.constraint(equalToConstant: parent.audioPlayerView.playqueueView.bounds.height)
+                    grabberView.isHidden = true
+                    closeButton.isHidden = true
+                    topConstraint = view.topAnchor.constraint(equalTo: parent.audioPlayerView.playqueueView.topAnchor)
+                    bottomConstraint = view.bottomAnchor.constraint(equalTo: parent.audioPlayerView.playqueueView.bottomAnchor)
+                    darkOverlayView.isHidden = true
                 } else {
                     topConstraint = view.topAnchor.constraint(equalTo: parent.view.bottomAnchor)
                     heightConstraint = nil
                     bottomConstraint = view.bottomAnchor.constraint(equalTo: parent.view.bottomAnchor)
                 }
             }
+
             parent.view.addSubview(view)
+
+            let leadingAnchor: NSLayoutXAxisAnchor
+            let trailingAnchor: NSLayoutXAxisAnchor
+            if #available(iOS 11.0, *) {
+                let safeArea: UILayoutGuide
+                if let miniPlayerView = miniPlayerView {
+                    safeArea = miniPlayerView.safeAreaLayoutGuide
+                } else {
+                    safeArea = parent.view.safeAreaLayoutGuide
+                }
+
+                leadingAnchor = safeArea.leadingAnchor
+                trailingAnchor = safeArea.trailingAnchor
+            } else {
+                leadingAnchor = parent.view.leadingAnchor
+                trailingAnchor = parent.view.trailingAnchor
+            }
+
             constraints = [
-                view.leadingAnchor.constraint(equalTo: parent.view.leadingAnchor),
-                view.trailingAnchor.constraint(equalTo: parent.view.trailingAnchor)
+                view.leadingAnchor.constraint(equalTo: leadingAnchor),
+                view.trailingAnchor.constraint(equalTo: trailingAnchor)
             ]
 
             if let topConstraint = topConstraint {
@@ -260,6 +291,15 @@ class QueueViewController: UIViewController {
         })
     }
 
+    func dismissFromAudioPlayer() {
+        UIView.animate(withDuration: animationDuration, animations: {
+            self.view.alpha = 0.0
+        }, completion: { _ in
+            self.view.removeFromSuperview()
+            self.removeFromParent()
+        })
+    }
+
     @objc private func dismissPlayqueue() {
         if let parent = parent {
             var newY: CGFloat = view.frame.height
@@ -410,9 +450,13 @@ private extension QueueViewController {
     }
 
     @objc private func dismissView() {
+        guard let delegate = delegate as? VideoPlayerViewController else {
+            return
+        }
+
         dismiss(animated: true) {
             [weak self] in
-            self?.delegate?.queueViewControllerDidDisappear(self)
+            delegate.queueViewControllerDidDisappear(self)
         }
     }
 }
@@ -423,7 +467,7 @@ extension QueueViewController: UICollectionViewDelegateFlowLayout {
     func collectionView(_ collectionView: UICollectionView,
                         layout collectionViewLayout: UICollectionViewLayout,
                         sizeForItemAt indexPath: IndexPath) -> CGSize {
-        return CGSize(width: collectionView.frame.width - (sidePadding * 2), height: cellHeight)
+        return MediaCollectionViewCell.cellSizeForWidth(collectionView.frame.width - (sidePadding * 2))
     }
 
     func collectionView(_ collectionView: UICollectionView,
@@ -504,6 +548,10 @@ extension QueueViewController: UICollectionViewDelegate, MediaCollectionViewCell
 
         return nil
     }
+
+    func mediaCollectionViewCellGetModel() -> MediaLibraryBaseModel? {
+        return nil
+    }
 }
 
 // MARK: - UIScrollViewDelegate
@@ -554,7 +602,10 @@ extension QueueViewController: UICollectionViewDataSource {
             return
         }
         mediaList.lock()
-        let currentMedia = mediaList.media(at: UInt(sourceIndexPath.row))
+        guard let currentMedia = mediaList.media(at: UInt(sourceIndexPath.row)) else {
+            mediaList.unlock()
+            return
+        }
         if mediaList.removeMedia(at: UInt(sourceIndexPath.row)) {
             mediaList.insert(currentMedia, at: UInt(destinationIndexPath.row))
         }
@@ -578,7 +629,7 @@ extension QueueViewController: UICollectionViewDataSource {
 
         var media: VLCMedia?
 
-        cell.thumbnailWidth.constant = cell.frame.height
+        cell.thumbnailWidth.constant = cell.getDefaultConstant()
 
         cell.ignoreThemeDidChange = true
         cell.setTheme(to: PresentationTheme.darkTheme)
@@ -592,16 +643,17 @@ extension QueueViewController: UICollectionViewDataSource {
         let isSelected = playbackService.currentlyPlayingMedia == media
         updateCollectionViewCellApparence(cell, isSelected: isSelected)
 
-        guard let safeMedia = media else {
-            assertionFailure("QueueViewController: cellForItemAt: Failed to fetch media")
+        guard let safeURL = media?.url else {
+            assertionFailure("QueueViewController: cellForItemAt: Failed to fetch media url")
             return cell
         }
-        if let media = medialibraryService.fetchMedia(with: safeMedia.url) {
+        if let media = medialibraryService.fetchMedia(with: safeURL) {
             cell.media = media
-        } else if let media = medialibraryService.medialib.addExternalMedia(withMrl: safeMedia.url) {
+        } else if let media = medialibraryService.medialib.addExternalMedia(withMrl: safeURL) {
             cell.media = media
         }
         cell.newLabel.isHidden = true
+
         return cell
     }
 }
@@ -609,11 +661,11 @@ extension QueueViewController: UICollectionViewDataSource {
 // MARK: - VLCMediaListDelegate
 
 extension QueueViewController: VLCMediaListDelegate {
-    func mediaList(_ aMediaList: VLCMediaList!, mediaAdded media: VLCMedia!, at index: UInt) {
+    func mediaList(_ aMediaList: VLCMediaList, mediaAdded media: VLCMedia, at index: UInt) {
         reload()
     }
 
-    func mediaList(_ aMediaList: VLCMediaList!, mediaRemovedAt index: UInt) {
+    func mediaList(_ aMediaList: VLCMediaList, mediaRemovedAt index: UInt) {
         reload()
     }
 }
